const express = require('express');
const ejs = require('ejs');
const bodyParser = require('body-parser');
const mysql = require('mysql');
const session = require('express-session');
const path = require('path');
const bcrypt = require('bcryptjs');

const app = express();
app.use(express.static(path.join(__dirname, 'public')));


// Login system code
const connectionLogin = mysql.createConnection({
    host: 'localhost',
    user: 'root',
    password: '',
    database: 'nodelogin'
});

app.use(session({
    secret: 'secret',
    resave: true,
    saveUninitialized: true
}));
app.use(express.json());
app.use(express.urlencoded({ extended: true }));
app.use(express.static(path.join(__dirname, 'static')));

app.get('/login', function (req, res) {
    // Render login template
    res.sendFile(path.join(__dirname + '/views/pages/login.html'));
});

app.post('/auth', function (req, res) {
    // Capture the input fields
    let username = req.body.username;
    let password = req.body.password;

    if (username && password) {
        connectionLogin.query('SELECT * FROM accounts WHERE username = ?', [username], function (error, results, fields) {
            if (error) throw error;

            if (results.length > 0) {
                // User found, compare passwords using bcrypt
                bcrypt.compare(password, results[0].password, function (err, bcryptResult) {
                    if (err) throw err;

                    if (bcryptResult) {
                        req.session.loggedin = true;
                        req.session.username = username;
                        res.redirect('/');
                    } else {
                        res.send('Incorrect Username and/or Password!');
                    }
                    res.end();
                });
            } else {
                res.send('Incorrect Username and/or Password!');
                res.end();
            }
        });
    } else {
        res.send('Please enter both Username and Password!');
        res.end();
    }
});

app.get('/dashboard', function (req, res) {
    if (req.session.loggedin) {
        res.send('Welcome back, ' + req.session.username + '!');
    } else {
        res.send('Please login to view this page!');
    }
    res.end();
});

// Create a MySQL connection pool
const pool = mysql.createPool({
    host: "localhost",
    user: "root",
    password: "",
    database: "node_j-tech"
});

app.set('view engine', 'ejs');
app.use(bodyParser.urlencoded({ extended: true }));
app.use(session({ secret: "secret" }));

// Middleware for database connection
app.use((req, res, next) => {
    pool.getConnection((err, connection) => {
        if (err) return next(err);
        req.connection = connection;
        next();
    });
});

// Middleware to release database connection
app.use((req, res, next) => {
    req.connection.release();
    next();
});

// Middleware to calculate total and store in session
app.use((req, res, next) => {
    // Ensure req.session.cart is defined before calculating total
    if (req.session.cart) {
        const total = calculateTotal(req.session.cart);
        req.session.total = total;
        console.log('Total calculated:', total);
    }
    next();
});


// Function to check if a product is in the cart
function isProductInCart(cart, ID) {
    for (let i = 0; i < cart.length; i++) {
        if (cart[i].id == ID) {
            return true;
        }
    }
    return false;
}

// Function to calculate the total in the cart
function calculateTotal(cart) {
    let total = 0;
    for (let i = 0; i < cart.length; i++) {
        // If we are offering a discounted price
        if (cart[i].Sale_price && !isNaN(cart[i].Sale_price) && !isNaN(cart[i].Quantity)) {
            total += cart[i].Sale_price * cart[i].Quantity;
        } else if (!isNaN(cart[i].Price) && !isNaN(cart[i].Quantity)) {
            total += cart[i].Price * cart[i].Quantity;
        }
    }
    return total.toFixed(2); // Convert total to string with two decimal places
}

// Route to fetch top-selling products for the home page
app.get('/', function (req, res, next) {
    req.connection.query("SELECT * FROM products WHERE Quantity <= 25", function (err, result) {
        if (err) return next(err);
        res.render('pages/index', { result: result });
    });
});

// Route to fetch all products without quantity filtering
app.get('/products', function (req, res, next) {
    req.connection.query("SELECT * FROM products", function (err, result) {
        if (err) return next(err);

        // Fetch categories for the sidebar
        req.connection.query("SELECT DISTINCT Category FROM products", function (err, categories) {
            if (err) return next(err);

            // Render the products page with products and categories
            res.render('pages/products', { result: result, categories: categories.map(category => category.Category) });
        });
    });
});

// Route to add a product to the cart
app.post('/add_to_cart', function (req, res) {
    var ID = req.body.id;
    var Name = req.body.Name;
    var Price = req.body.Price;
    var Sale_price = req.body.Sale_price;
    var Quantity = req.body.Quantity;
    var Image = req.body.Image;
    var Product = { id: ID, Name: Name, Price: Price, Sale_price: Sale_price, Quantity: Quantity, Image: Image };

    if (req.session.cart) {
        var cart = req.session.cart;

        if (!isProductInCart(cart, ID)) {
            cart.push(Product);
        }
    } else {
        req.session.cart = [Product];
    }

    // Calculate total
    const total = calculateTotal(req.session.cart);
    req.session.total = total;

    // Return to cart page
    res.redirect('/product/' + ID);
});

// Route to display the cart
app.get('/cart', function (req, res, next) {
    var cart = req.session.cart;
    var total = req.session.total;

    console.log('Total in /cart route:', total);

    // Pass the loggedin variable to the cart template
    res.render('pages/cart', { cart: cart, total: total, loggedin: req.session.loggedin });
});


// Route to fetch categories for the home page
app.get('/home', function(req, res, next) {
    req.connection.query("SELECT DISTINCT Category FROM products", function(err, categories) {
        if (err) return next(err);
        res.render('pages/index', { categories: categories.map(category => category.Category), result: [] });
    });
});

app.get('/search', function (req, res, next) {
    var query = req.query.query;
    var category = req.query.category;

    if (!query && !category) {
        res.redirect('/products');
        return;
    }

    var sql;
    var params;

    if (query && category) {
        sql = "SELECT * FROM products WHERE (Name LIKE ? OR Category LIKE ? OR Type LIKE ?) AND Category = ?";
        params = [`%${query}%`, `%${query}%`, `%${query}%`, category];
    } else if (query) {
        sql = "SELECT * FROM products WHERE Name LIKE ? OR Category LIKE ? OR Type LIKE ?";
        params = [`%${query}%`, `%${query}%`, `%${query}%`];
    } else if (category) {
        sql = "SELECT * FROM products WHERE Category = ?";
        params = [category];
    }
    req.connection.query(sql, params, function (err, result) {
        if (err) return next(err);
        res.render('pages/search', { result: result, query: query, category: category });
    });
});

// Route to display individual product details
app.get('/product/:id', function (req, res, next) {
    var productId = req.params.id;

    var sql = "SELECT * FROM products WHERE ID = ?";
    var params = [productId];

    req.connection.query(sql, params, function (err, result) {
        if (err) return next(err);
        // Check if the product is found
        if (result.length === 0) {
            res.status(404).send('Product not found');
            return;
        }
        // Render the product details page
        res.render('pages/single_product', { product: result[0] });
    });
});

app.post('/remove_from_cart', function (req, res) {
    var cart = req.session.cart;
    var productIdToRemove = req.body.id;

    // Find the index of the product to remove
    var indexToRemove = -1;
    for (var i = 0; cart && i < cart.length; i++) {
        if (cart[i].id === productIdToRemove) {
            indexToRemove = i;
            break;
        }
    }

    // Remove the product if found
    if (indexToRemove !== -1) {
        cart.splice(indexToRemove, 1);
        // Recalculate the total
        const total = calculateTotal(cart);
        req.session.total = total;
    }

    res.redirect('/cart'); // Redirect back to the cart page
});

// Route to handle checkout form submission
app.post('/process_checkout', function (req, res) {
    // Extract form data from req.body
    const name = req.body.name;
    const address = req.body.address;
    // Additional form fields go here

    // Process the order, store it in the database, etc.

    // Clear the cart after successful checkout
    req.session.cart = [];
    req.session.total = 0;

    // Redirect to a thank you or order confirmation page
    res.redirect('/thank_you');
});

// Route to render the checkout page
app.get('/checkout', function (req, res) {
    if (req.session.loggedin) {
        // Render the checkout page if the user is logged in
        res.render('pages/checkout.ejs', { total: req.session.total });
    } else {
        // Redirect to the login page if the user is not logged in
        res.redirect('/login');
    }
});


// Route to render the thank you page
app.get('/thank_you', function (req, res) {
    // Render the thank you page
    res.render('pages/thank_you.ejs');
});

// Start the server
const PORT = process.env.PORT || 8080;  // Use environment variable if available
app.listen(PORT, function () {
    console.log(`Server is running on port ${PORT}`);
});

// Route to render the registration page
app.get('/register', function (req, res) {
    res.sendFile(path.join(__dirname + '/views/pages/register.html'));
});

// Route to handle registration form submission
app.post('/register', function (req, res) {
    let username = req.body.username;
    let password = req.body.password;
    let email = req.body.email;

    if (username && password && email) {
        // Check if the username is already taken
        connectionLogin.query('SELECT * FROM accounts WHERE username = ?', [username], function (error, results) {
            if (error) throw error;

            if (results.length > 0) {
                res.send('Username already exists. Please choose a different username.');
            } else {
                // Hash the password
                bcrypt.hash(password, 10, function (err, hash) {
                    if (err) throw err;

                    // Insert the new user into the database with the hashed password
                    connectionLogin.query('INSERT INTO accounts (username, password, email) VALUES (?, ?, ?)', [username, hash, email], function (error) {
                        if (error) throw error;
                        res.send('Registration successful! You can now <a href="/login">login</a>.');
                    });
                });
            }
        });
    } else {
        res.send('Please enter both username, password, and email.');
    }
});
